/*====================================+=====================================+
! File CPlaylist.h                    ! Copyright (C) 2016-2018 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! matelefree is free software: you can redistribute it and/or modify it     !
! under the terms of the GNU General Public License as published by the     !
! Free Software Foundation, either version 3 of the License, or any later   !
! version.                                                                  !
! matelefree is distributed in the hope that it will be useful, but WITHOUT !
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or     !
! FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for !
! more details.                                                             !
! You should have received a copy of the GNU General Public License along   !
! with matelefree. If not, see <http://www.gnu.org/licenses/>.              !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#pragma once
/*-------------------------------------------------------------------------*/
#include <vlc/vlc.h>
#include <vector>
#include <wx/treectrl.h>
#include <wx/listctrl.h>
#include <wx/srchctrl.h>
#include <wx/bannerwindow.h>
#include "matelefree.h"
/*-------------------------------------------------------------------------*/
class CPlaylist ;
class CPlaylistFilter ;
class CPlaylistFbtvoOk ;
class CPlaylistUser ;
class CPlaylistSearch ;
class CPlaylistLive ;
class CPlaylistNode ;
class CPlaylistUserNode ;
class CVlcMedia ;
class CChannel ;
class CChannelLive ;
class CPlSubWinLive ;
class CPlSubWinChOnair ;
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
wxDECLARE_EVENT( EVT_AST_CONTEXT_MENU, wxCommandEvent ) ;
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
using t_vec_pl = std::vector< CPlaylist * > ;

/*-------------------------------------------------------------------------*/
struct str_pk_info
{
   /*----------------------------------------------------------------------*/
   str_pk_info( const wxString &s_name, const wxString &s_label )
              : m_s_name( s_name ), m_s_label( s_label )
   {  ; }
   /*----------------------------------------------------------------------*/
   wxString  m_s_name ;
   wxString  m_s_label ;
   CPlaylist *m_pl = nullptr ;
   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CPlaylist : public wxListCtrl
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CPlaylist( wxWindow *parent,
                 const wxString &s_name, const wxString &s_label,
                 const wxString &s_tree_path
               ) ;
      //~CPlaylist() ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      enum e_id_pl
      {  ID_PL_LOADING_STREAMS,
         ID_PL_PLAY_ACTIVE,
         ID_PL_PLAY_OTHER,
         ID_PL_PLAY_WITH_VLC,
         ID_PL_COPY_URL,
         ID_PL_STREAM_MIN,
         ID_PL_STREAM_MAX = ID_PL_STREAM_MIN + 100,
         ID_PL_NB
      } ;
      /*-------------------------------------------------------------------*/
      static constexpr const char *m_st_co_p_c_cfg_path = "/playlist" ;
      /*-------------------------------------------------------------------*/
      wxCriticalSection m_csec_context_menu ;

   /*----------------------------------------------------------------------*/
   protected :
      /*-------------------------------------------------------------------*/
      wxMenu *m_context_menu = nullptr ;
      /*--( Known "id" if any )--------------------------------------------*/
      e_pl_known m_pl_known = PK_NB ;
      /*--( Image entry in "bopl"'s image list )---------------------------*/
      int m_i_il_pos = -1 ;
      /*--( Channels displayed by this playlist )--------------------------*/
      mtf::t_vec_string m_vs_channel ;
      /*--( Location in the directory tree )-------------------------------*/
      wxString m_s_tree_path ;
      /*--( The tree item "holding" this playlist )------------------------*/
      wxTreeItemId m_ti ;
      /*--( Internal name and name to be displayed )-----------------------*/
      wxString m_s_name ;
      wxString m_s_label ;
      bool m_boo_text_long = true ; // Channels listed with long names ?
      /*--( Playlist column width )----------------------------------------*/
      static int m_st_i_col_width ;
      /*-------------------------------------------------------------------*/
      bool m_boo_accept_ch    = false ; // Accept channels from other pl ?
      bool m_boo_give_ch      = false ; // Move channels to other pl ?
      bool m_boo_rename_ch    = false ; // Channels can be renamed ?
      bool m_boo_delete_ch    = false ; // Channels can be deleted ?
      bool m_boo_cut_paste_ch = false ; // Channels can be cut / pasted ?
      bool m_boo_order_ch     = false ; // Channels can be reordered ?
      bool m_boo_browser_ch   = false ; // Channels "openable" in a browser ?
      bool m_boo_deletable    = false ; // Playlist can be deleted  ?
      bool m_boo_renamable    = false ; // Playlist can be renamed  ?
      bool m_boo_movable      = false ; // Playlist can be move up & down ?
      bool m_boo_drag_in_progress = false ;
      /*--( Playlist info has to be saved in the config file ? )-----------*/
      bool m_boo_save_cfg = false ;
      /*--( The on air status is only shown (for now) for "Live" channels )*/
      virtual bool get_use_attr_onair() const { return( false ) ; }

   /*----------------------------------------------------------------------*/
   protected :
      /*-------------------------------------------------------------------*/
      virtual int OnGetItemImage( long l_item ) const override ;
      virtual wxString OnGetItemText( long l_item, long l_column
                                    ) const override ;
      virtual wxListItemAttr *OnGetItemAttr( long l_item ) const override ;
      void OnBeginDrag( wxListEvent &event ) ;
      void OnKeyDown( wxKeyEvent &event ) ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual CPlSubWinChOnair *get_win_onair() const { return( nullptr ) ; }
      /*-------------------------------------------------------------------*/
      void swap_vs_channel( mtf::t_vec_string &vs_channel )
      {  m_vs_channel.swap( vs_channel ) ; }
      const mtf::t_vec_string &get_vs_channel() const
      {  return( m_vs_channel ) ; }
      const int get_nb_channel() const { return( m_vs_channel.size() ) ; }
      e_pl_known get_pl_known() const { return( m_pl_known ) ; }
      bool get_text_long() const { return( m_boo_text_long ) ; }
      void set_text_long( bool boo_on ) { m_boo_text_long = boo_on ; }
      bool get_drag_in_progress() const { return( m_boo_drag_in_progress ) ;}
      void set_drag_in_progress( bool boo_on )
      {  m_boo_drag_in_progress = boo_on ; }
      const wxString &get_tree_path() const { return( m_s_tree_path ) ; }
      void set_tree_path( const wxString &s ) { m_s_tree_path = s ; }
      const wxTreeItemId &get_ti() const { return( m_ti ) ; }
      void set_ti( const wxTreeItemId &ti ) { m_ti = ti ; }
      const wxString &get_name() const { return( m_s_name ) ; }
      const wxString &get_label() const { return( m_s_label ) ; }
      void set_label( const wxString &s_val ) { m_s_label = s_val ; }
      bool get_accept_ch() const { return( m_boo_accept_ch ) ; }
      void set_accept_ch( bool boo_val ) { m_boo_accept_ch = boo_val ; }
      bool get_give_ch() const { return( m_boo_give_ch ) ; }
      void set_give_ch( bool boo_val ) { m_boo_give_ch = boo_val ; }
      bool get_rename_ch() const { return( m_boo_rename_ch ) ; }
      void set_rename_ch( bool boo_val ) { m_boo_rename_ch = boo_val ; }
      bool get_delete_ch() const { return( m_boo_delete_ch ) ; }
      void set_delete_ch( bool boo_val ) { m_boo_delete_ch = boo_val ; }
      bool get_cut_paste_ch() const { return( m_boo_cut_paste_ch ) ; }
      void set_cut_paste_ch( bool boo_val ) { m_boo_cut_paste_ch = boo_val ;}
      bool get_order_ch() const { return( m_boo_order_ch ) ; }
      void set_order_ch( bool boo_val ) { m_boo_order_ch = boo_val ; }
      bool get_browser_ch() const { return( m_boo_browser_ch ) ; }
      void set_browser_ch( bool boo_val ) { m_boo_browser_ch = boo_val ; }
      bool get_deletable() const { return( m_boo_deletable ) ; }
      void set_deletable( bool boo_val ) { m_boo_deletable = boo_val ; }
      bool get_renamable() const { return( m_boo_renamable ) ; }
      void set_renamable( bool boo_val ) { m_boo_renamable = boo_val ; }
      bool get_movable() const { return( m_boo_movable ) ; }
      void set_movable( bool boo_val ) { m_boo_movable = boo_val ; }
      bool get_save_cfg() const { return( m_boo_save_cfg ) ; }
      void set_save_cfg( bool boo_val ) { m_boo_save_cfg = boo_val ; }
      /*--( Window to attach to the tree )---------------------------------*/
      virtual wxWindow *get_main_window() { return( this ) ; }
      void show_no_window( bool boo_on = true )
      {  get_main_window()->Show( boo_on ) ; }
      /*-------------------------------------------------------------------*/
      void init_image() ;
      int get_image_il_pos() ;
      static int st_get_col_width() { return( m_st_i_col_width ) ; }
      static void st_set_col_width( int i_val ) { m_st_i_col_width = i_val ;}
      static CPlaylist *st_new_pl_source( const wxString &s_name,
                                          const wxString &s_label,
                                          const wxString &s_tree_path,
                                          mtf::t_vec_string &vs_channel
                                        ) ;
      static CPlaylist *st_new_pl_source( e_pl_known pl_known,
                                          const wxString &s_tree_path,
                                          mtf::t_vec_string &vs_channel
                                        ) ;
      static CPlaylistFilter  *st_new_pl_fbtvb() ;
      static CPlaylistFilter  *st_new_pl_fbtvo() ;
      static CPlaylistFbtvoOk *st_new_pl_fbtvo_ok(
                                               const wxString &s_channel_list
                                                 ) ;
      static CPlaylistNode *st_new_pl_live_node() ;
      static CPlaylistLive *st_new_pl_live( e_pl_known pl_known,
                                            wxConfigBase *p_conf
                                          ) ;
      static CPlaylistUserNode *st_new_pl_user_node() ;
      static CPlaylistUser *st_new_pl_user( const wxString &s_text,
                                            const wxString &s_channel_list
                                          ) ;
      static CPlaylistSearch *st_new_pl_search() ;
      void init_item_count() ;
      /*-------------------------------------------------------------------*/
      virtual void string_to_datas( const wxString &s_val ) ;
      virtual wxString datas_to_string() const ;
      void sort_by_name() ;
      bool copy_hil_to_clipboard() ;
      void clear()
      {  m_vs_channel.clear() ; }
      virtual void init_list()
      {  init_item_count() ; }
      bool cut_hil_to_clipboard() ;
      bool paste_from_clipboard() ;
      virtual int add_channel( const mtf::t_vec_string &vs_channel,
                               int i_ins_pos = -1
                             ) ;
      int add_channel_hil( CPlaylist *pl_src, int i_ins_pos = -1 ) ;
      int add_channel_played() ;
      virtual int del_channel( const mtf::t_vec_string &vs_channel ) ;
      virtual int del_channel_hil() ;
      void ren_channel_hil() ;
      void change_channel_name( const wxString &s_from,
                                const wxString &s_to
                              ) ;
      void swap_channel( int i_item1, int i_item2 ) ;
      void move_channel_hil( const int co_i_nb_move ) ;
      virtual void append_channel_from( const CPlaylist *pl ) ;
      void rename( const wxString &s_name ) ;
      void delete_cfg() ;
      CChannel *get_channel( long l_item ) const ;
      long get_channel_item( const wxString &s_name ) const ;
      /*-------------------------------------------------------------------*/
      void set_item_state( long l_item, long l_state )
      {  SetItemState( l_item, l_state, l_state ) ; }
      void set_item_focus( long l_item )
      {  set_item_state( l_item, wxLIST_STATE_FOCUSED ) ; }
      bool is_item_hil( long l_item ) const
      {  return( GetItemState( l_item, wxLIST_STATE_SELECTED
                             ) == wxLIST_STATE_SELECTED
               ) ;
      }
      long get_item_focus() const
      {  return( GetNextItem( -1, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED ) );}
      /*-------------------------------------------------------------------*/
      int get_nb_item() const { return( GetItemCount() ) ; }
      int get_nb_hil() const { return( GetSelectedItemCount() ) ; }
      /*--( If "from" is equal to -1 it means from start )-----------------*/
      long get_prev_item_hil( long l_item_from ) const
      {  return( GetNextItem( l_item_from,
                              wxLIST_NEXT_ABOVE, wxLIST_STATE_SELECTED
                            )
               ) ;
      }
      long get_next_item_hil( long l_item_from ) const
      {  return( GetNextItem( l_item_from,
                              wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED
                            )
               ) ;
      }
      long get_first_item_hil() const
      {  return( get_next_item_hil( -1 ) ) ; }
      long get_last_item_hil() const ;
      /*-------------------------------------------------------------------*/
      void set_item_visible( long l_item )
      {  EnsureVisible( l_item ) ; }
      /*--( -1 means all )-------------------------------------------------*/
      void set_item_hil( long l_item, bool boo_on )
      {  SetItemState( l_item, ( boo_on ? wxLIST_STATE_SELECTED : 0 ),
                       wxLIST_STATE_SELECTED
                     ) ;
      }
      void set_item_hil( const mtf::t_vec_string &vs_channel, bool boo_on ) ;
      int get_hil( mtf::t_vec_string &vs_channel ) const ;
      int get_hil( mtf::t_vec_int &vi_item ) const ;
      void set_item_active( long l_item ) ;
      void refresh_item( long l_item ) ;
      void refresh_item( const wxString &s_name ) ;
      void play_item( long l_item, bool boo_new_media = false ) ;
      void play_focus() ;
      void play_prev_hil() ;
      void play_next_hil() ;
      wxMenu *build_track_menu( CVlcMedia *p_media, libvlc_track_type_t type,
                                int i_id_start
                              ) const ;
      int get_track_id_by_pos( CVlcMedia *p_media, libvlc_track_type_t type,
                               int i_pos
                             ) const ;
      void refresh_streams( CChannel *p_channel ) ;
      virtual void add_streams_to_context_menu( CChannel *p_channel ) ;
      void add_streams_to_context_menu( const wxString &s_channel_name ) ;
      void context_menu( wxListEvent &event ) ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CPlaylistNode : public CPlaylist
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CPlaylistNode( wxWindow *parent,
                     const wxString &s_name, const wxString &s_label,
                     const wxString &s_tree_path
                   )
                   : CPlaylist( parent, s_name, s_label, s_tree_path )
      {  ; }

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual void init_list() override ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CPlaylistNodeWithPanel : public CPlaylistNode
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CPlaylistNodeWithPanel( wxWindow *parent,
                             const wxString &s_name, const wxString &s_label,
                              const wxString &s_tree_path
                            ) ;

   /*----------------------------------------------------------------------*/
   protected :
      /*-------------------------------------------------------------------*/
      wxPanel    *m_panel = nullptr ;
      wxBoxSizer *m_bsz   = nullptr ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual wxWindow *get_main_window() override
      {  return( m_panel ) ; }

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CPlaylistLiveNode : public CPlaylistNodeWithPanel
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CPlaylistLiveNode( wxWindow *parent,
                         const wxString &s_name, const wxString &s_label,
                         const wxString &s_tree_path
                       ) ;

   /*----------------------------------------------------------------------*/
   private :
      /*-------------------------------------------------------------------*/
      CPlSubWinChOnair *m_win_onair = nullptr ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual CPlSubWinChOnair *get_win_onair() const override
      {  return( m_win_onair ) ; }
      /*--( The on air status is only shown in "Live" playlists )----------*/
      virtual bool get_use_attr_onair() const override { return( true ) ; }

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CPlaylistUserNode : public CPlaylistNodeWithPanel
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CPlaylistUserNode( wxWindow *parent,
                         const wxString &s_name, const wxString &s_label,
                         const wxString &s_tree_path
                       ) ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      void append_channel_from( const CPlaylist *pl ) ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CPlaylistFilter : public CPlaylist
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      using t_fnc_filter = bool (*)( const CChannel *p_channel ) ;
      /*-------------------------------------------------------------------*/
      CPlaylistFilter( wxWindow *parent,
                       const wxString &s_name,
                       const wxString &s_label,
                       const wxString &s_tree_path,
                       CPlaylist      *pl_src,
                       t_fnc_filter   fnc_filter
                     )
                     : CPlaylist( parent, s_name, s_label, s_tree_path ),
                       m_pl_src( pl_src ),
                       m_fnc_filter( fnc_filter )
      {
         /*----------------------------------------------------------------*/
         m_boo_text_long = false ;
         /*----------------------------------------------------------------*/
      }

   /*----------------------------------------------------------------------*/
   protected :
      /*-------------------------------------------------------------------*/
      CPlaylist    *m_pl_src ; // Playlist on which the filtering is done
      t_fnc_filter m_fnc_filter ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual void init_list() override ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CPlaylistWithPanel : public CPlaylist
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CPlaylistWithPanel( wxWindow *parent,
                          const wxString &s_name,
                          const wxString &s_label,
                          const wxString &s_tree_path
                        ) ;

   /*----------------------------------------------------------------------*/
   protected :
      /*-------------------------------------------------------------------*/
      wxPanel    *m_panel = nullptr ;
      wxBoxSizer *m_bsz   = nullptr ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual wxWindow *get_main_window() override
      {  return( m_panel ) ; }

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CPlaylistLive : public CPlaylistWithPanel
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CPlaylistLive( wxWindow *parent,
                     const wxString &s_name, const wxString &s_label,
                     const wxString &s_tree_path
                   ) ;
      virtual ~CPlaylistLive()
      {  ; }

   /*----------------------------------------------------------------------*/
   private :
      /*-------------------------------------------------------------------*/
      CPlSubWinLive *m_win_action = nullptr ;

   /*----------------------------------------------------------------------*/
   public :
      /*--( The on air status is only shown in "Live" playlists )----------*/
      virtual bool get_use_attr_onair() const override { return( true ) ; }
      /*-------------------------------------------------------------------*/
      CPlSubWinLive *get_win_action() const { return( m_win_action ) ; }
      virtual CPlSubWinChOnair *get_win_onair() const override ;
      /*-------------------------------------------------------------------*/
      virtual wxString get_source() const = 0 ;
      virtual wxString get_url_format() const = 0 ;
      virtual wxString build_url( const wxString &s_id ) const = 0 ;
      virtual wxString get_url_example() const = 0 ;
      void string_to_datas( const wxString &s_channel_list ) ;
      virtual wxString datas_to_string() const override ;
      void init_access_onair() ;
      /*-------------------------------------------------------------------*/
      virtual int extract_data_from_url( const wxString &s_url,
                                         wxString &s_id,
                                         wxString &s_name,
                                         wxString *p_s_err_msg
                                       ) const = 0 ;
      virtual CChannelLive *new_channel( const wxString &s_id,
                                         const wxString &s_name
                                       ) const = 0 ;
      CChannel *find_channel( const wxString &s_id ) const ;
      int add_channel( CChannelLive *p_channel ) ;
      virtual int del_channel_hil() override ;
      virtual void add_streams_to_context_menu( CChannel *p_channel )
              override ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CPlaylistDailymotion : public CPlaylistLive
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CPlaylistDailymotion( wxWindow *parent,
                            const wxString &s_name, const wxString &s_label,
                            const wxString &s_tree_path
                          )
                 : CPlaylistLive( parent, s_name, s_label, s_tree_path )
      {  ; }
      virtual ~CPlaylistDailymotion()
      {  ; }

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual wxString get_source() const override
      {  return( CChannelDailymotion::st_get_pl_source() ) ; }
      virtual wxString get_url_format() const override
      {  return( CChannelDailymotion::st_get_url_format() ) ; }
      virtual wxString build_url( const wxString &s_id ) const override
      {  return( CChannelDailymotion::st_build_url( s_id ) ) ; }
      virtual wxString get_url_example() const override
      {  return( CChannelDailymotion::st_get_url_example() ) ; }
      virtual int extract_data_from_url( const wxString &s_url,
                                         wxString &s_id,
                                         wxString &s_name,
                                         wxString *p_s_err_msg
                                       ) const
      {  return( CChannelDailymotion::st_extract_data_from_url(
                                             s_url, s_id, s_name, p_s_err_msg
                                                              )
               ) ;
      }
      virtual CChannelLive *new_channel( const wxString &s_id,
                                         const wxString &s_name
                                       ) const
      {  return( new CChannelDailymotion( s_id, s_name ) ) ; }

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CPlaylistTwitch : public CPlaylistLive
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CPlaylistTwitch( wxWindow *parent,
                       const wxString &s_name, const wxString &s_label,
                       const wxString &s_tree_path
                     )
                     : CPlaylistLive( parent, s_name, s_label, s_tree_path )
      {  ; }
      virtual ~CPlaylistTwitch()
      {  ; }

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual wxString get_source() const override
      {  return( CChannelTwitch::st_get_pl_source() ) ; }
      virtual wxString get_url_format() const override
      {  return( CChannelTwitch::st_get_url_format() ) ; }
      virtual wxString build_url( const wxString &s_id ) const override
      {  return( CChannelTwitch::st_build_url( s_id ) ) ; }
      virtual wxString get_url_example() const override
      {  return( CChannelTwitch::st_get_url_example() ) ; }
      virtual int extract_data_from_url( const wxString &s_url,
                                         wxString &s_id,
                                         wxString &s_name,
                                         wxString *p_s_err_msg
                                       ) const
      {  return( CChannelTwitch::st_extract_data_from_url(
                                             s_url, s_id, s_name, p_s_err_msg
                                                         )
               ) ;
      }
      virtual CChannelLive *new_channel( const wxString &s_id,
                                         const wxString &s_name
                                       ) const
      {  return( new CChannelTwitch( s_id, s_name ) ) ; }

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CPlaylistYoutube : public CPlaylistLive
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CPlaylistYoutube( wxWindow *parent,
                        const wxString &s_name, const wxString &s_label,
                        const wxString &s_tree_path
                      )
                     : CPlaylistLive( parent, s_name, s_label, s_tree_path )
      {  ; }
      virtual ~CPlaylistYoutube()
      {  ; }

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual wxString get_source() const override
      {  return( CChannelYoutube::st_get_pl_source() ) ; }
      virtual wxString get_url_format() const override
      {  return( CChannelYoutube::st_get_url_format() ) ; }
      virtual wxString build_url( const wxString &s_id ) const override
      {  return( CChannelYoutube::st_build_url( s_id ) ) ; }
      virtual wxString get_url_example() const override
      {  return( CChannelYoutube::st_get_url_example() ) ; }
      virtual int extract_data_from_url( const wxString &s_url,
                                         wxString &s_id,
                                         wxString &s_name,
                                         wxString *p_s_err_msg
                                       ) const
      {  return( CChannelYoutube::st_extract_data_from_url(
                                             s_url, s_id, s_name, p_s_err_msg
                                                          )
                ) ;
      }
      virtual CChannelLive *new_channel( const wxString &s_id,
                                         const wxString &s_name
                                       ) const
      {  return( new CChannelYoutube( s_id, s_name ) ) ; }

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CPlaylistUstream : public CPlaylistLive
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CPlaylistUstream( wxWindow *parent,
                        const wxString &s_name, const wxString &s_label,
                        const wxString &s_tree_path
                      )
                     : CPlaylistLive( parent, s_name, s_label, s_tree_path )
      {  ; }
      virtual ~CPlaylistUstream()
      {  ; }

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual wxString get_source() const override
      {  return( CChannelUstream::st_get_pl_source() ) ; }
      virtual wxString get_url_format() const override
      {  return( CChannelUstream::st_get_url_format() ) ; }
      virtual wxString build_url( const wxString &s_id ) const override
      {  return( CChannelUstream::st_build_url( s_id ) ) ; }
      virtual wxString get_url_example() const override
      {  return( CChannelUstream::st_get_url_example() ) ; }
      virtual int extract_data_from_url( const wxString &s_url,
                                         wxString &s_id,
                                         wxString &s_name,
                                         wxString *p_s_err_msg
                                       ) const
      {  return( CChannelUstream::st_extract_data_from_url(
                                             s_url, s_id, s_name, p_s_err_msg
                                                          )
                ) ;
      }
      virtual CChannelLive *new_channel( const wxString &s_id,
                                         const wxString &s_name
                                       ) const
      {  return( new CChannelUstream( s_id, s_name ) ) ; }

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CPlaylistPeriscope : public CPlaylistLive
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CPlaylistPeriscope( wxWindow *parent,
                          const wxString &s_name, const wxString &s_label,
                          const wxString &s_tree_path
                        )
                      : CPlaylistLive( parent, s_name, s_label, s_tree_path )
      {  ; }
      virtual ~CPlaylistPeriscope()
      {  ; }

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual wxString get_source() const override
      {  return( CChannelPeriscope::st_get_pl_source() ) ; }
      virtual wxString get_url_format() const override
      {  return( CChannelPeriscope::st_get_url_format() ) ; }
      virtual wxString build_url( const wxString &s_id ) const override
      {  return( CChannelPeriscope::st_build_url( s_id ) ) ; }
      virtual wxString get_url_example() const override
      {  return( CChannelPeriscope::st_get_url_example() ) ; }
      virtual int extract_data_from_url( const wxString &s_url,
                                         wxString &s_id,
                                         wxString &s_name,
                                         wxString *p_s_err_msg
                                       ) const
      {  return( CChannelPeriscope::st_extract_data_from_url(
                                             s_url, s_id, s_name, p_s_err_msg
                                                            )
               ) ;
      }
      virtual CChannelLive *new_channel( const wxString &s_id,
                                         const wxString &s_name
                                       ) const
      {  return( new CChannelPeriscope( s_id, s_name ) ) ; }

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CPlaylistMixer : public CPlaylistLive
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CPlaylistMixer( wxWindow *parent,
                      const wxString &s_name, const wxString &s_label,
                      const wxString &s_tree_path
                    )
                  : CPlaylistLive( parent, s_name, s_label, s_tree_path )
      {  ; }
      virtual ~CPlaylistMixer()
      {  ; }

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual wxString get_source() const override
      {  return( CChannelMixer::st_get_pl_source() ) ; }
      virtual wxString get_url_format() const override
      {  return( CChannelMixer::st_get_url_format() ) ; }
      virtual wxString build_url( const wxString &s_id ) const override
      {  return( CChannelMixer::st_build_url( s_id ) ) ; }
      virtual wxString get_url_example() const override
      {  return( CChannelMixer::st_get_url_example() ) ; }
      virtual int extract_data_from_url( const wxString &s_url,
                                         wxString &s_id,
                                         wxString &s_name,
                                         wxString *p_s_err_msg
                                       ) const
      {  return( CChannelMixer::st_extract_data_from_url(
                                             s_url, s_id, s_name, p_s_err_msg
                                                        )
               ) ;
      }
      virtual CChannelLive *new_channel( const wxString &s_id,
                                         const wxString &s_name
                                       ) const
      {  return( new CChannelMixer( s_id, s_name ) ) ; }

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CPlaylistSearch : public CPlaylistWithPanel
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CPlaylistSearch( wxWindow *parent,
                       const wxString &s_name, const wxString &s_label,
                       const wxString &s_tree_path
                     ) ;

   /*----------------------------------------------------------------------*/
   private :
      /*-------------------------------------------------------------------*/
      wxSearchCtrl *m_search = nullptr ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      void exec_search() ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CPlaylistUser : public CPlaylistWithPanel
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CPlaylistUser( wxWindow *parent,
                     const wxString &s_name,
                     const wxString &s_label,
                     const wxString &s_tree_path
                   ) ;

   /*----------------------------------------------------------------------*/
   private :
      /*-------------------------------------------------------------------*/
      wxBannerWindow *m_banner = nullptr ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual void init_list() override ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CPlaylistFbtvoOk : public CPlaylistWithPanel
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CPlaylistFbtvoOk( wxWindow *parent,
                        const wxString &s_name, const wxString &s_label,
                        const wxString &s_tree_path
                      ) ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
using t_vec_playlist     = std::vector< CPlaylist * >     ;
using t_vec_playlist_it  = t_vec_playlist::iterator       ;
using t_vec_playlist_cit = t_vec_playlist::const_iterator ;
/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                          End of file CPlaylist.h                          !
+==========================================================================*/
