/*====================================+=====================================+
! File CChecksum.cpp                  ! Copyright (C) 2016-2018 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! matelefree is free software: you can redistribute it and/or modify it     !
! under the terms of the GNU General Public License as published by the     !
! Free Software Foundation, either version 3 of the License, or any later   !
! version.                                                                  !
! matelefree is distributed in the hope that it will be useful, but WITHOUT !
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or     !
! FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for !
! more details.                                                             !
! You should have received a copy of the GNU General Public License along   !
! with matelefree. If not, see <http://www.gnu.org/licenses/>.              !
+---------------------------------------------------------------------------+
!                                                                           !
!           Checksum computation: MD5, SHA1, SHA256 & CRC32                 !
!                                                                           !
+-------+-------------------------------------------------------------------+
! Notes !                                                                   !
+-------+                                                                   !
! This code is based on information found at these addresses:               !
!                                                                           !
! MD5:   http://en.wikipedia.org/wiki/Md5                                   !
!                                                                           !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#include "CApp.h"
#include "CChecksum.h"
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
bool CChecksumMD5::m_st_boo_init_done = false ;
wxUint32 CChecksumMD5::m_st_tb_dw_r[ 64 ]     ;
wxUint32 CChecksumMD5::m_st_tb_dw_k[ 64 ]     ;

/*-------------------------------------------------------------------------*/
void CChecksum::result_to_string( wxString &s_result ) const
{
   /*----------------------------------------------------------------------*/
   s_result.clear() ;
   s_result.reserve( m_i_result_len * 2 ) ;
   /*----------------------------------------------------------------------*/
   for( int i_pos = 0 ; i_pos < m_i_result_len ; ++i_pos )
   {  s_result += wxString::Format( "%02x", m_tb_uc_result[ i_pos ] ) ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CChecksum::calc( const unsigned char *p_uc_buffer, size_t sz_buffer_len,
                      unsigned char *p_uc_result
                    )
{
   /*----------------------------------------------------------------------*/
   init() ;
   add_data( p_uc_buffer, sz_buffer_len ) ;
   finalize() ;
   /*----------------------------------------------------------------------*/
   copy_result( p_uc_result ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CChecksum::calc( const char *p_c_buffer, unsigned char *p_uc_result )
{
   /*----------------------------------------------------------------------*/
   calc( ( unsigned char * )p_c_buffer, strlen( p_c_buffer ), p_uc_result ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString CChecksum::calc( const wxString &s_in )
{
   /*----------------------------------------------------------------------*/
   wxCharBuffer chb_in( s_in.mb_str() ) ;
   /*----------------------------------------------------------------------*/
   init() ;
   add_data( ( unsigned char * )chb_in.data(), chb_in.length() ) ;
   finalize() ;
   /*----------------------------------------------------------------------*/
   wxString s_result ;
   result_to_string( s_result ) ;
   /*----------------------------------------------------------------------*/
   return( s_result ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CChecksumMD5::init()
{
   /*--( Done only once: during first instantiation )----------------------*/
   if( !m_st_boo_init_done )
   {
      /*--------------------------------------------------------------------+
      ! It is useless to set this as "static" because the associated code   !
      ! is executed only once                                               !
      +--------------------------------------------------------------------*/
      wxUint32 tb_dw_init_r[ 4 ][ 4 ]
             = { { 7, 12, 17, 22 }, { 5,  9, 14, 20 },
                 { 4, 11, 16, 23 }, { 6, 10, 15, 21 }
               } ;
      /*-------------------------------------------------------------------*/
      for( int i_num = 0 ; i_num < 4 ; ++i_num )
      {  /*----------------------------------------------------------------*/
         for( int i_rep = 0 ; i_rep < 4 ; ++i_rep )
         {  /*-------------------------------------------------------------*/
            memcpy( &m_st_tb_dw_r[ i_num * 16 + i_rep * 4 ],
                    tb_dw_init_r[ i_num ], sizeof( tb_dw_init_r[ 0 ] )
                  ) ;
            /*-------------------------------------------------------------*/
         }
         /*----------------------------------------------------------------*/
      }

      /*-------------------------------------------------------------------*/
      for( int i_num = 0 ; i_num < 64 ; ++i_num )
      {    m_st_tb_dw_k[ i_num ]
         = ( wxUint32 )floor(   fabs( sin( ( double )i_num + 1 ) )
                              * 4294967296.0
                            ) ;
      }
      /*-------------------------------------------------------------------*/
      m_st_boo_init_done = true ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   m_i_result_len = 0 ;
   /*----------------------------------------------------------------------*/
   m_ull_total_size = 0 ;
   m_i_not_treated_buffer_len = 0 ;
   m_dw_h0 = wxUINT32_SWAP_ON_BE( 0x67452301 ) ;
   m_dw_h1 = wxUINT32_SWAP_ON_BE( 0xEFCDAB89 ) ;
   m_dw_h2 = wxUINT32_SWAP_ON_BE( 0x98BADCFE ) ;
   m_dw_h3 = wxUINT32_SWAP_ON_BE( 0x10325476 ) ;
   /*----------------------------------------------------------------------*/
}

/*--( The buffer is supposed to be 64 bytes long )-------------------------*/
void CChecksumMD5::add_data_64bytes( const unsigned char *p_uc_buffer,
                                     CChecksumMD5::e_sltt sltt
                                   )
{
   /*----------------------------------------------------------------------*/
   wxUint32 *p_dw = ( wxUint32 * )p_uc_buffer ;
   wxUint32 dw_a  = m_dw_h0 ;
   wxUint32 dw_b  = m_dw_h1 ;
   wxUint32 dw_c  = m_dw_h2 ;
   wxUint32 dw_d  = m_dw_h3 ;
   wxUint32 dw_f            ;
   wxUint32 dw_g            ;
   wxUint32 dw_temp         ;

   /*----------------------------------------------------------------------*/
   for( int i_num = 0 ; i_num < 64 ; ++i_num )
   {
      /*-------------------------------------------------------------------*/
      if( i_num < 16 )
      {  dw_f = ( ( dw_b & dw_c ) | ( ~dw_b & dw_d ) ) ;
         dw_g = i_num ;
      }
      else
      if( i_num < 32 )
      {  dw_f = ( ( dw_d & dw_b ) | ( ~dw_d & dw_c ) ) ;
         dw_g = ( 5 * i_num + 1 ) % 16 ;
      }
      else
      if( i_num < 48 )
      {  dw_f = ( dw_b ^ dw_c ^ dw_d ) ;
         dw_g = ( 3 * i_num + 5 ) % 16 ;
      }
      else
      {  dw_f = ( dw_c ^ ( dw_b | ~dw_d ) ) ;
         dw_g = ( 7 * i_num ) % 16 ;
      }
      /*-------------------------------------------------------------------*/
      dw_temp = dw_d ;
      dw_d    = dw_c ;
      dw_c    = dw_b ;
      dw_b   += leftrotate(   dw_a + dw_f + m_st_tb_dw_k[ i_num ]
                            + wxINT32_SWAP_ON_BE( p_dw[ dw_g ] ),
                            m_st_tb_dw_r[ i_num ]
                          ) ;
      dw_a    = dw_temp ;
      /*-------------------------------------------------------------------*/
   }

   /*----------------------------------------------------------------------*/
   m_dw_h0 += dw_a ;
   m_dw_h1 += dw_b ;
   m_dw_h2 += dw_c ;
   m_dw_h3 += dw_d ;
   /*----------------------------------------------------------------------*/
   if( sltt == DO_SUM_LENGTH_TO_TOTAL )
   {  m_ull_total_size += 64 ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CChecksumMD5::add_data( const unsigned char *p_uc_buffer,
                             size_t sz_buffer_len
                           )
{
   /*--( Something left ? )------------------------------------------------*/
   if( m_i_not_treated_buffer_len > 0 )
   {
      /*-------------------------------------------------------------------*/
      size_t sz_size_to_add
           = wxMin( sz_buffer_len, 64 - m_i_not_treated_buffer_len ) ;

      /*--( Fill the buffer up ... )---------------------------------------*/
      memcpy( &m_tb_uc_not_treated_buffer[ m_i_not_treated_buffer_len ],
              p_uc_buffer, sz_size_to_add
            ) ;
      m_i_not_treated_buffer_len += sz_size_to_add ;
      p_uc_buffer   += sz_size_to_add ;
      sz_buffer_len -= sz_size_to_add ;

      /*--( And if filled, treat it )--------------------------------------*/
      if( m_i_not_treated_buffer_len == 64 )
      {
         /*----------------------------------------------------------------*/
         add_data_64bytes( m_tb_uc_not_treated_buffer ) ;
         m_i_not_treated_buffer_len = 0 ;
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
   }

   /*--( Treat the full 64 bytes parts )-----------------------------------*/
   while( sz_buffer_len >= 64 )
   {
      /*-------------------------------------------------------------------*/
      add_data_64bytes( p_uc_buffer ) ;
      p_uc_buffer   += 64 ;
      sz_buffer_len -= 64 ;
      /*-------------------------------------------------------------------*/
   }

   /*--( Keep the remaining bytes )----------------------------------------*/
   if( sz_buffer_len > 0 )
   {
      /*-------------------------------------------------------------------*/
      memcpy( m_tb_uc_not_treated_buffer, p_uc_buffer, sz_buffer_len ) ;
      m_i_not_treated_buffer_len = sz_buffer_len ;
      /*-------------------------------------------------------------------*/
   }

   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CChecksumMD5::flush_data_add_size( e_stsz stsz )
{
   /*----------------------------------------------------------------------*/
   wxUint32 dw_nb_bits ;

   /*--( This will be the "final" data length )----------------------------*/
   m_ull_total_size += m_i_not_treated_buffer_len ;
   /*--( Add the last "1" bit. There is buffer space )---------------------*/
   m_tb_uc_not_treated_buffer[ m_i_not_treated_buffer_len++ ] = 0x80 ;

   /*--( Enough place to add the length ? )--------------------------------*/
   if( m_i_not_treated_buffer_len >= 56 )
   {
      /*--( No, fill this buffer and add it )------------------------------*/
      memset( &m_tb_uc_not_treated_buffer[ m_i_not_treated_buffer_len ],
              '\0', 64 - m_i_not_treated_buffer_len
            ) ;
      /*--( Add this one without incrementing the total length )-----------*/
      add_data_64bytes( m_tb_uc_not_treated_buffer,
                        DONT_SUM_LENGTH_TO_TOTAL
                      ) ;
      m_i_not_treated_buffer_len = 0 ;
      /*-------------------------------------------------------------------*/
   }

   /*--( Filling up and leaving enough place for the size )----------------*/
   memset( &m_tb_uc_not_treated_buffer[ m_i_not_treated_buffer_len ],
           '\0', 56 - m_i_not_treated_buffer_len
         ) ;

   /*--( Original length in bits (not bytes) )-----------------------------*/
   m_ull_total_size *= 8 ;
   /*--( Size stored depending on the endianess )--------------------------*/
   dw_nb_bits = ( stsz == STORE_SIZE_LE
                  ? wxUINT32_SWAP_ON_BE( m_ull_total_size.GetLo() )
                  : wxUINT32_SWAP_ON_LE( m_ull_total_size.GetHi() )
                ) ;
   memcpy( &m_tb_uc_not_treated_buffer[ 56 ], &dw_nb_bits, 4 )  ;
   /*----------------------------------------------------------------------*/
   dw_nb_bits = ( stsz == STORE_SIZE_LE
                  ? wxUINT32_SWAP_ON_BE( m_ull_total_size.GetHi() )
                  : wxUINT32_SWAP_ON_LE( m_ull_total_size.GetLo() )
                ) ;
   memcpy( &m_tb_uc_not_treated_buffer[ 60 ], &dw_nb_bits, 4 )  ;

   /*--( Add it )----------------------------------------------------------*/
   add_data_64bytes( m_tb_uc_not_treated_buffer, DONT_SUM_LENGTH_TO_TOTAL ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CChecksumMD5::finalize()
{
   /*--( Treat remaining data and add size )-------------------------------*/
   flush_data_add_size( STORE_SIZE_LE ) ;

   /*--( Store the binary result )-----------------------------------------*/
   wxUint32 *p_dw_result = ( wxUint32 * )m_tb_uc_result ;
   p_dw_result[ 0 ] = wxUINT32_SWAP_ON_BE( m_dw_h0 ) ;
   p_dw_result[ 1 ] = wxUINT32_SWAP_ON_BE( m_dw_h1 ) ;
   p_dw_result[ 2 ] = wxUINT32_SWAP_ON_BE( m_dw_h2 ) ;
   p_dw_result[ 3 ] = wxUINT32_SWAP_ON_BE( m_dw_h3 ) ;
   m_i_result_len = 4 * 4 ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                         End of file CChecksum.cpp                         !
+==========================================================================*/
